#include "common.h"
#include "DispMessage.h"
#include "adapIntgrtn.h"

using namespace System;
using namespace CheckSgmEDistriMod;
using namespace adapInt;

namespace cmmn {


	double * Y;
	int * X;
	double ** XTable;
	double igm_aA, igm_bA, igm_aB, igm_bB, igm_aE, igm_bE;
	double * samples_sgmA2, * samples_sgmB2, * samples_sgmE2, * samples_Rho;
	double ** rho2t_ni, * rho2t_LB, * rho2t_UB, * rho2t_Med;
	

	int N, NA, NB;
	int MaxNi;

	double LogProb( int na, int nb, double ** xtable, double mu, double * paramA, double * paramB, 
					double sgmA2, double sgmB2, double sgmE2 ){
		double v = 0.0;
		v = -(igm_aA + 1.0) * Math::Log(sgmA2) - (igm_bA / sgmA2)
			-(igm_aB + 1.0) * Math::Log(sgmB2) - (igm_bB / sgmB2)
			-(igm_aE + 1.0) * Math::Log(sgmE2) - (igm_bE / sgmE2);
		double sgmA = Math::Sqrt(sgmA2);
		double sgmB = Math::Sqrt(sgmB2);
		double sgmE = Math::Sqrt(sgmE2);
		for (int t = 0; t < na; t++)
			v += -Math::Log(sgmA) - 0.5 * sqr(paramA[t] / sgmA);
		for (int i = 0; i < nb; i++)
			v += -Math::Log(sgmB) - 0.5 * sqr(paramB[i] / sgmB);
		for (int t = 0; t < na; t++)
			for (int i = 0; i < nb; i++)
				v += -Math::Log(sgmE) - 0.5 * sqr((xtable[t][i] - (mu + paramA[t] + paramB[i])) / sgmE);

		return v;
	}
	
	void MCMC::Init(String ^ msg, int nJmpRN, int vNSimu, int vna, int vnb, double ** vxtable, double vmu, double * vparamA, double * vparamB,
						double vsgmA2, double vsgmB2, double vsgmE2, double vgen_SgmMu, double * vgen_paramA, double * vgen_paramB,
						double vgen_sgmA2, double vgen_sgmB2, double vgen_sgmE2){
					MnMssg = msg;
					NSimu = vNSimu;
					rn = new RNP2to191();
					for (int i = 0; i < nJmpRN; i++)
						rn->Jump2toj(100);
					NA = vna;
					NB = vnb;
					XTable = new double*[NA];
					for (int t = 0; t < NA; t++){
						XTable[t] = new double[NB];
						for (int i = 0; i < NB; i++)
							XTable[t][i] = vxtable[t][i];
					}
					mu = new double[NSimu+1];
					mu[0] = vmu;
					paramA = new double*[NSimu+1];
					for (int j = 0; j <= NSimu; j++)
						paramA[j] = new double[NA];
					for (int t = 0; t < NA; t++)
						paramA[0][t] = vparamA[t];
					paramB = new double*[NSimu+1];
					for (int j = 0; j <= NSimu; j++)
						paramB[j] = new double[NB];
					for (int i = 0; i < NB; i++)
						paramB[0][i] = vparamB[i];
					sgmA2 = new double[NSimu+1];
					sgmA2[0] = vsgmA2;
					sgmB2 = new double[NSimu+1];
					sgmB2[0] = vsgmB2;
					sgmE2 = new double[NSimu+1];
					sgmE2[0] = vsgmE2;

					gen_SgmMu = vgen_SgmMu;
					gen_paramA = new double[NA];
					for (int t = 0; t < NA; t++)
						gen_paramA[t] = vgen_paramA[t];
					gen_paramB = new double[NB];
					for (int i = 0; i < NB; i++)
						gen_paramB[i] = vgen_paramB[i];
					gen_sgmA2 = vgen_sgmA2;
					gen_sgmB2 = vgen_sgmB2;
					gen_sgmE2 = vgen_sgmE2;

					cnt_mu = 0;
					cnt_paramA = new int[NA];
					for (int t = 0; t < NA; t++)
						cnt_paramA[t] = 0;
					cnt_paramB = new int[NB];
					for (int i = 0; i < NB; i++)
						cnt_paramB[i] = 0;
					cnt_sgmA2 = 0;
					cnt_sgmB2 = 0;
					cnt_sgmE2 = 0;

					rho2_t = new double[NSimu+1];
					rho2_t[0] = sgmA2[0] / (sgmA2[0] + (sgmE2[0] / NB));
	}

	void MCMC::do_mcmc(){
		DispMessage cfm;
		cfm.Show();
		cfm.SetMsg(MnMssg);
		for (int imcmc = 0; imcmc < NSimu; imcmc++){
			if((imcmc % 1000) == 0){
				cfm.ShowMessage( "\n imcmc = " + imcmc.ToString() + "/" + NSimu.ToString());
			}

			double y = rn->normalMS( mu[imcmc], gen_SgmMu );
			double num = LogProb( NA, NB, XTable, y, paramA[imcmc], paramB[imcmc], sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
			double den = LogProb( NA, NB, XTable, mu[imcmc], paramA[imcmc], paramB[imcmc], sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
			double a = Math::Exp( num - den );
			if (rn->uni() < a){
				cnt_mu++;
				mu[imcmc+1] = y;
			}else{
				mu[imcmc+1] = mu[imcmc];
			}
			double * tA0 = new double[NA], * tA1 = new double[NA];
			for (int t = 0; t < NA; t++)
				tA0[t] = tA1[t] = paramA[imcmc][t];
			for (int t = 0; t < NA; t++){
				double y = rn->normalMS( tA0[t], gen_paramA[t] );
				tA1[t] = y;
				double num = LogProb( NA, NB, XTable, mu[imcmc+1], tA1, paramB[imcmc], sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
				double den = LogProb( NA, NB, XTable, mu[imcmc+1], tA0, paramB[imcmc], sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
				double a = Math::Exp( num - den );
				if (rn->uni() < a){
					cnt_paramA[t]++;
					tA0[t] = tA1[t];
				}else{
					tA1[t] = tA0[t];
				}
			}
			for (int t = 0; t < NA; t++)
				paramA[imcmc+1][t] = tA1[t];
			delete [] tA0;
			delete [] tA1;
			double * tB0 = new double[NB], * tB1 = new double[NB];
			for (int i = 0; i < NB; i++)
				tB0[i] = tB1[i] = paramB[imcmc][i];
			for (int i = 0; i < NB; i++){
				double y = rn->normalMS( tB0[i], gen_paramB[i] );
				tB1[i] = y;
				double num = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], tB1, sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
				double den = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], tB0, sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
				double a = Math::Exp( num - den );
				if (rn->uni() < a){
					cnt_paramB[i]++;
					tB0[i] = tB1[i];
				}else{
					tB1[i] = tB0[i];
				}
			}
			for (int i = 0; i < NB; i++)
				paramB[imcmc+1][i] = tB1[i];
			delete [] tB0;
			delete [] tB1;
			y = rn->normalMS(sgmA2[imcmc], gen_sgmA2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], y, sgmB2[imcmc], sgmE2[imcmc] );
				den = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], sgmA2[imcmc], sgmB2[imcmc], sgmE2[imcmc] );
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmA2++;
				sgmA2[imcmc+1] = y;
			}else{
				sgmA2[imcmc+1] = sgmA2[imcmc];
			}
			y = rn->normalMS(sgmB2[imcmc], gen_sgmB2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], sgmA2[imcmc+1], y, sgmE2[imcmc] );
				den = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], sgmA2[imcmc+1], sgmB2[imcmc], sgmE2[imcmc] );
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmB2++;
				sgmB2[imcmc+1] = y;
			}else{
				sgmB2[imcmc+1] = sgmB2[imcmc];
			}
			y = rn->normalMS(sgmE2[imcmc], gen_sgmE2);
			if(y <= 0.0){
				a = -1.0;
			}else{
				num = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], sgmA2[imcmc+1], sgmB2[imcmc+1], y );
				den = LogProb( NA, NB, XTable, mu[imcmc+1], paramA[imcmc+1], paramB[imcmc+1], sgmA2[imcmc+1], sgmB2[imcmc+1], sgmE2[imcmc] );
				a = Math::Exp(num - den);
			}
			if(rn->uni() < a){
				cnt_sgmE2++;
				sgmE2[imcmc+1] = y;
			}else{
				sgmE2[imcmc+1] = sgmE2[imcmc];
			}

			rho2_t[imcmc+1] = sgmA2[imcmc+1] / (sgmA2[imcmc+1] + (sgmE2[imcmc+1] / NB));
		}

		cfm.Close();
	}



	
	double g_a;

	double func1( double x ){
		double v = Math::Pow( x, g_a - 1.0 ) * Math::Exp( -x ); 
		return v;
	}

	double func2( double x ){
		double v = Math::Pow( -Math::Log( x ), g_a - 1.0 );
		return v;
	}


	double gamma( double a ){
			g_a = a;
			AdaptiveGL agl;
			double v = agl.integral( 0.0, 1.0, func1 ) + agl.integral( 0.0, Math::Exp(-1.0), func2 );
			return v;
	}


	double k_smoother( double x, double * samples, int N, double sgm ){
		double v = 0.0;
		for (int i = 0; i < N; i++)
			if (Math::Abs(x - samples[i]) < 4 * sgm){
				v += Math::Exp( -0.5 * sqr((x - samples[i]) / sgm));
			}
			return v / (N * sgm * Math::Sqrt(2.0 * Math::Atan(1.0) * 4.0));
	}


}
