#ifndef COMMON_CK
#define COMMON_CK

#include "RNP2to191.h"

using namespace System;
using namespace rnp2to191;

namespace cmmn {

	extern double * Y;
	extern int * X;
	extern double ** XTable;
	extern double igm_aA, igm_bA, igm_aB, igm_bB, igm_aE, igm_bE;
	extern double * samples_sgmA2, * samples_sgmB2, * samples_sgmE2, * samples_Rho;
	extern double ** rho2t_ni, * rho2t_LB, * rho2t_UB, * rho2t_Med;
	
	extern int N, NA, NB;
	extern int MaxNi;

	inline double sqr( double v ){
			return  v * v;
	}

	double LogProb(String ^ msg, int vNSimu, int na, int nb, double ** xtable, double mu, double * paramA, double * paramB, 
					double sgmA2, double sgmB2, double sgmE2, double vgen_SgmMu, double * vgen_paramA, double * vgen_paramB,
						double vgen_sgmA2, double vgen_sgmB2, double vgen_sgmE2 );

	ref class MCMC {
		public:
			String ^ MnMssg;
			int NSimu;
			int NA, NB;
			double ** XTable;
			RNP2to191 * rn;
			double * mu, ** paramA, ** paramB, * sgmA2, * sgmB2, * sgmE2;
			double gen_SgmMu, * gen_paramA, * gen_paramB, gen_sgmA2, gen_sgmB2, gen_sgmE2;
			int cnt_mu, * cnt_paramA, * cnt_paramB, cnt_sgmA2, cnt_sgmB2, cnt_sgmE2;
			double * rho2_t;

			MCMC() { }

			void clearMe(){
				if (XTable != nullptr){
					for (int t = 0; t < NA; t++)
						delete [] XTable[t];
					delete [] XTable;
				}
				delete [] mu;
				if (paramA != nullptr){
					for (int j = 0; j <= NSimu; j++)
						delete [] paramA[j];
					delete [] paramA;
				}
				if (paramB != nullptr){
					for (int j = 0; j <= NSimu; j++)
						delete [] paramB[j];
					delete [] paramB;
				}
				if (sgmA2 != nullptr) 
					delete [] sgmA2;
				if (sgmB2 != nullptr)
					delete [] sgmB2;
				if (sgmE2 != nullptr)
					delete [] sgmE2;

				if(gen_paramA != nullptr) delete [] gen_paramA;
				if(gen_paramB != nullptr) delete [] gen_paramB;
				if(cnt_paramA != nullptr) delete [] cnt_paramA;
				if(cnt_paramB != nullptr) delete [] cnt_paramB;

				if(rho2_t != nullptr) delete [] rho2_t;
			}

			virtual ~MCMC(){ clearMe(); }
			!MCMC(){ clearMe(); }

			void Init(String ^ msg, int nJmpRN, int vNSimu, int vna, int vnb, double ** vxtable, double vmu, double * vparamA, double * vparamB,
						double vsgmA2, double vsgmB2, double vsgmE2, double vgen_SgmMu, double * vgen_paramA, double * vgen_paramB,
						double vgen_sgmA2, double vgen_sgmB2, double vgen_sgmE2);

			void do_mcmc();
	};

	double gamma( double a );

	double k_smoother( double x, double * samples, int N, double sgm );

}

#endif